#include <stdio.h>

#include <zephyr/shell/shell.h>
#include <zephyr/logging/log.h>
#include <zephyr/device.h>
#ifdef CONFIG_DT_HAS_SEMTECH_SX1509B_ENABLED
#include <zephyr/drivers/gpio/gpio_sx1509b.h>
#endif
#include <zephyr/drivers/gpio.h>

#include "mosh_print.h"
#include "io_expander_shell.h"

LOG_MODULE_REGISTER(io_expander_shell, 4);

#ifdef CONFIG_DT_HAS_SEMTECH_SX1509B_ENABLED
#define USERLED_1 DT_GPIO_PIN(DT_NODELABEL(led1), gpios)
#else
#define USERLED_1 (0)
#endif

static const struct device *io_expander;

#ifdef CONFIG_DT_HAS_SEMTECH_SX1509B_ENABLED
static int toggle_led_sx1509b(void) {
	io_expander = DEVICE_DT_GET(DT_NODELABEL(io_expander));

	if (!device_is_ready(io_expander)) {
		LOG_ERR("io_expander: device not ready.\n");
		return -1;
	}

	static bool is_configured = false;
	if (!is_configured) {
		int ret = sx1509b_led_intensity_pin_configure(io_expander, USERLED_1);
		if (ret < 0) {
			LOG_ERR("USERLED_1: configure failed");

			return -1;
		}

		is_configured = true;
	}

	// The SX1509B driver has special LED intensity functions for controlling
	// an LED, but the expected values are a range from 0 to 255.
	static uint8_t toggle_value = 0;

	sx1509b_led_intensity_pin_set(io_expander, USERLED_1, toggle_value);

	// Make it be either off (0) or on (255).
	toggle_value ^= 255;

	return 0;
}
#else
static int toggle_led_tca9534(void) {
	int ret = 0;
	io_expander = DEVICE_DT_GET(DT_NODELABEL(io_expander));

	if (!device_is_ready(io_expander)) {
		LOG_ERR("io_expander: device not ready.\n");
		return;
	}

	static bool is_configured = false;
	if (!is_configured) {
		ret = gpio_pin_configure(io_expander, USERLED_1, GPIO_OUTPUT);
		if (ret < 0) {
			LOG_ERR("USERLED_1: configure failed");

			return -1;
		}

		is_configured = true;
	}

	ret = gpio_pin_toggle(io_expander, USERLED_1);
	if (ret < 0) {
		LOG_ERR("Failed to toggle USERLED_1 pin");

		return -1;
	}

	return 0;
}
#endif

static int toggle_led(void) {
	int ret;
#ifdef CONFIG_DT_HAS_SEMTECH_SX1509B_ENABLED
	ret = toggle_led_sx1509b();
#else
	ret = toggle_led_tca9534();
#endif

	return ret;
}

int io_expander_shell(const struct shell *shell, size_t argc, char **argv) {
	mosh_print("io_expander command interface.");

	mosh_print("Toggling USERLED_1");

	toggle_led();

	return 0;
}